import L from 'leaflet'
import PropTypes from 'prop-types';
import React, { useEffect, useRef, useState } from 'react'

import "leaflet/dist/leaflet.css";
import icon from "leaflet/dist/images/marker-icon.png";
import iconShadow from "leaflet/dist/images/marker-shadow.png";

let DefaultIcon = L.icon({
iconUrl: icon,
shadowUrl: iconShadow,
});

export default function MapObject( { object } ) {
    console.log('map-object:MapObject: object.size: ' + object.size);

    /*center map at Node: Umweltzentrum Braunschweig (3650315388)*/
    const mapCenter = [52.2671189, 10.5221905];

    const mapRef = useRef();

    useEffect(() => {

	//base layer
	const baseLayerOsm = L.tileLayer(
	    "http://{s}.tile.osm.org/{z}/{x}/{y}.png",
	    {
                attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors',
		maxZoom: 18,
		scrollWheelZoom: false,
	    }
	);

	//map
        const map = L.map(
	    mapRef.current,
	    {attributionControl: true,
	     layers : [baseLayerOsm]}
	).setView(
	    mapCenter,
	    10
	);

	const rryTitleKeys = new Array();
	const rryLayerValues = new Array();
	//iterate over osm rsp objects
	for (const [title, elements] of object) {
	    console.log(`The value for key ${title} is ${elements}`);

	    //add title
	    rryTitleKeys.push(title);

	    //add layer
	//marker array
	const rryMrkr = elements.map( function( value, key ) {

	    //console.log('map-object:MapObject:map() value: ' +JSON.stringify(value));

	    //get entity properties
	    const id = value.id;
	    const type = value.type;
	    const name = value.tags.name;
	    let lat, lon = null;
	    if ( type === "node"){
		lat = value.lat;
		lon = value.lon;
	    } else if ( type === "way" || type === "relation" ) {
		lat = value.center.lat;
		lon = value.center.lon;
	    } else {
		console.error("map-entities: OSM type (" + type + ") NOT known");
	    }

	    //verify entity properties
	    if(lat !== undefined && lat !== null && lon !== undefined && lon !== null ){
		//TODO
		/*
		var marker = L.marker([lat, lon]).addTo(map);
		const popupText ='name: <b>' + name + '</b><br />id: ' + id + '<br />type: ' + type;
		marker.bindPopup(popupText);

		*/
		L.Marker.prototype.options.icon = DefaultIcon;
		const marker = L.marker(
		    [lat, lon],
		    {alt: 'marker name: ' + name}
		);
		const popupText ='name: <b>' + name + '</b><br />lat: ' + lat + '<br />lon: ' + lon;
		//console.log('popupText: ' + popupText);
		marker.bindPopup(popupText);
		return marker
	    } else {
		console.error('map-entities: lat or lon undefined or null');
	    }

	});
	console.log('rryMrkr: ' + rryMrkr.length);

	//layer group
	const overlayMrkr = L.layerGroup(rryMrkr);
	    rryLayerValues.push(overlayMrkr);
	}
	console.log('rryLayerValues.lngth: ' + rryLayerValues.length);
	console.log('rryTitleKeys.lngth: ' + rryTitleKeys.length);

	const overlayMaps = new Object();
	for (let i = 0; i < rryTitleKeys.length; i++) {
	    const ttl = rryTitleKeys[i];
	    const lyr = rryLayerValues[i];
	    console.log('i: ' + i + ', ttl: ' + ttl);
	    overlayMaps[ttl] = lyr;
	}
	//ERROR console.log('overlayMaps: ' + JSON.stringify(overlayMaps));
	
	//layout control
	const baseMaps = {
	    "OpenStreetMap": baseLayerOsm
	};
	const layerControl = L.control.layers(baseMaps, overlayMaps).addTo(map);

	// unmount map function
	//You should unmount the function in react.js to remove the existing map.
	return () => map.remove();
    }, []);    

    if ( object !== undefined && object !== null && object.size > 0 ) {
	return (
	    <>
		<p>MapObject</p>
		<div
		    style={{padding: 0, margin: 0, width: "75%", height: "23vh",}}
		    ref={el => mapRef.current = el}>
		</div>
	    </>
	);
    }else{
	//TODO Why does mapRef have to be present?
	return (
            <>
		<p>MapObject loading...</p>
		<div
		    ref={el => mapRef.current = el}>
		</div>
            </>
	);
    }

};
//https://github.com/azaharyan/react-leaflet-example

MapObject.propTypes = {
    array: PropTypes.array
};
